package com.tzld.longarticle.recommend.server.service;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.data.Record;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.collect.Lists;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.tzld.longarticle.recommend.server.common.CommonThreadPoolExecutor;
import com.tzld.longarticle.recommend.server.common.enums.aigc.ProduceContentAuditStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.AccountBusinessTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.FeishuRobotIdEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.CrawlerBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.growth.NewPushMessageCallbackMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.*;
import com.tzld.longarticle.recommend.server.model.entity.aigc.CrawlerPlan;
import com.tzld.longarticle.recommend.server.model.entity.aigc.ProducePlan;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishAccount;
import com.tzld.longarticle.recommend.server.model.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.model.entity.crawler.GetOffVideoCrawler;
import com.tzld.longarticle.recommend.server.model.entity.crawler.LongArticlesVideo;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.*;
import com.tzld.longarticle.recommend.server.model.param.ArticleFindSourceParam;
import com.tzld.longarticle.recommend.server.model.param.TitleHisCacheParam;
import com.tzld.longarticle.recommend.server.remote.ODPSManager;
import com.tzld.longarticle.recommend.server.repository.aigc.ProducePlanRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.GetOffVideoCrawlerRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.LongArticlesVideoRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.*;
import com.tzld.longarticle.recommend.server.repository.model.PushMessageCallbackExample;
import com.tzld.longarticle.recommend.server.service.recommend.ArticleCategoryService;
import com.tzld.longarticle.recommend.server.service.recommend.ArticlePromotionService;
import com.tzld.longarticle.recommend.server.service.recommend.ArticleService;
import com.tzld.longarticle.recommend.server.service.recommend.recall.RecallService;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.LarkRobotUtil;
import com.tzld.longarticle.recommend.server.util.feishu.FeishuMessageSender;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.tzld.longarticle.recommend.server.common.constant.TimeConstant.MILLISECOND_DAY;
import static com.tzld.longarticle.recommend.server.common.constant.TimeConstant.MILLISECOND_HOUR;

@Service
@Slf4j
public class XxlJobService {

    @Autowired
    private AigcBaseMapper aigcBaseMapper;
    @Autowired
    private CrawlerBaseMapper crawlerBaseMapper;
    @Autowired
    private LongArticleBaseMapper longArticleBaseMapper;
    @Autowired
    private GetOffVideoCrawlerRepository getOffVideoCrawlerRepository;
    @Autowired
    private LongArticlesVideoRepository longArticlesVideoRepository;
    @Autowired
    private GetOffVideoArticleRepository getOffVideoArticleRepository;
    @Autowired
    private LongArticlesMatchVideoRepository longArticlesMatchVideoRepository;
    @Autowired
    private LongArticlesReadRateRepository longArticlesReadRateRepository;
    @Autowired
    private LongArticlesRootSourceIdRepository longArticlesRootSourceIdRepository;
    @Autowired
    private NewPushMessageCallbackMapper newPushMessageCallbackMapper;
    @Autowired
    private ODPSManager odpsManager;
    @Autowired
    private ArticleService articleService;
    @Autowired
    private ArticleCategoryService articleCategoryService;
    @Autowired
    private ArticlePromotionService articlePromotionService;
    @Autowired
    private ArticleTitleHisCacheRepository articleTitleHisCacheRepository;
    @Autowired
    private RecallService recallService;
    @Autowired
    private ProducePlanRepository producePlanRepository;

    ExecutorService thread = new CommonThreadPoolExecutor(
            5,
            5,
            0L, TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(),
            new ThreadFactoryBuilder().setNameFormat("XXLJOB-%d").build(),
            new ThreadPoolExecutor.AbortPolicy());

    @ApolloJsonValue("${touliu.account.ghIds:[\"gh_93e00e187787\", \"gh_ac43e43b253b\", \"gh_68e7fdc09fe4\",\"gh_77f36c109fb1\", \"gh_b181786a6c8c\", \"gh_1ee2e1b39ccf\"]}")
    private List<String> touliuAccountGhIds;

    @XxlJob("checkPublishPlan")
    public ReturnT<String> checkPublishPlan(String param) {
        Long todayStart = DateUtils.getTodayStart();
        List<NotPublishPlan> notPublishPlan = aigcBaseMapper.getNotPublishPlan(todayStart);
        for (NotPublishPlan publishPlan : notPublishPlan) {
            long now = System.currentTimeMillis();
            int nowHour = DateUtils.getHourByTimestamp(now / 1000);
            int nowMinute = DateUtils.getMinuteByTimestamp(now / 1000);
            if (!StringUtils.hasText(publishPlan.getPublishPushTime())) {
                continue;
            }
            LocalTime time = LocalTime.parse(publishPlan.getPublishPushTime());
            int planHour = time.getHour();
            int planMinute = time.getMinute();
            if (nowHour - planHour == 1 && nowMinute > planMinute) {
                sendFeishuPublishPlanNotPushWarn(publishPlan);
            } else if (nowHour - planHour > 1) {
                sendFeishuPublishPlanNotPushWarn(publishPlan);
            }
        }
        return ReturnT.SUCCESS;
    }

    @Getter
    @Setter
    @Accessors(chain = true)
    public static class PlanErrorMsg {
        private String date;
        List<PlanErrorMsgDetail> errorMsgDetails;
    }

    @Getter
    @Setter
    @Accessors(chain = true)
    public static class PlanErrorMsgDetail {
        private String accountName;
        private String errorMsg;
    }

    private void sendFeishuPublishPlanNotPushWarn(NotPublishPlan publishPlan) {
        Long todayStart = DateUtils.getTodayStart();
        List<PublishAccount> publishAccounts = aigcBaseMapper.getPublishAccounts(publishPlan.getPlanId(), todayStart);
        List<String> accountNames;
        if (CollectionUtil.isNotEmpty(publishAccounts)) {
            accountNames = publishAccounts.stream().map(PublishAccount::getName).collect(Collectors.toList());
        } else {
            return;
        }
        PlanErrorMsg planErrorMsg = JSONObject.parseObject(publishPlan.getErrorMsg(), PlanErrorMsg.class);
        if (planErrorMsg == null) {
            return;
        } else if (CollectionUtil.isEmpty(planErrorMsg.getErrorMsgDetails())) {
            return;
        }
        Iterator<PlanErrorMsgDetail> iterator = planErrorMsg.getErrorMsgDetails().iterator();
        while (iterator.hasNext()) {
            PlanErrorMsgDetail errorMsgDetail = iterator.next();
            if (errorMsgDetail.getErrorMsg().contains("45028")
                    || errorMsgDetail.getErrorMsg().contains("48004")
                    || errorMsgDetail.getErrorMsg().contains("50002")) {
                iterator.remove();
                accountNames.remove(errorMsgDetail.getAccountName());
            }
        }
        if (CollectionUtil.isEmpty(planErrorMsg.getErrorMsgDetails())
                || CollectionUtil.isEmpty(accountNames)) {
            return;
        }
        FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.RECOMMEND.getRobotId(),
                "发布计划超1H未发送完成\n"
                        + "发布计划ID: " + publishPlan.getPlanId() + "\n"
                        + "发布计划名称: " + publishPlan.getPlanName() + "\n"
                        + "预计数量: " + publishPlan.getExpectCount() + "\n"
                        + "实际数量: " + publishPlan.getPublishCount() + "\n"
                        + "发送失败账号: " + JSONObject.toJSONString(accountNames) + "\n"
                        + "发布计划失败原因: " + JSONObject.toJSONString(planErrorMsg) + "\n"
                        + "发布时间: " + publishPlan.getPublishPushTime() + "\n"
                        + "<at user_id=\"all\">所有人</at> ");
    }

    @XxlJob("checkPublishPlanAccount")
    public ReturnT<String> checkPublishPlanAccount(String param) {
        List<PublishPlanAccountNotifyDTO> notifyAccounts = aigcBaseMapper.checkPublishPlanAccount();
        Map<String, List<PublishPlanAccountNotifyDTO>> notifyAccountMap = notifyAccounts.stream()
                .collect(Collectors.groupingBy(PublishPlanAccountNotifyDTO::getPlanId));
        for (Map.Entry<String, List<PublishPlanAccountNotifyDTO>> entry : notifyAccountMap.entrySet()) {
            String planId = entry.getKey();
            List<PublishPlanAccountNotifyDTO> list = entry.getValue();
            if (CollectionUtil.isEmpty(list)) {
                continue;
            }
            String planName = list.get(0).getPlanName();
            List<String> accountNames = list.stream().map(PublishPlanAccountNotifyDTO::getAccountName).collect(Collectors.toList());
            log.info("checkPublishPlanAccount planId: {}, planName: {}, accountNames: {}", planId, planName, JSONObject.toJSONString(accountNames));
            FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.RECOMMEND.getRobotId(),
                    "发布计划开启账号未开启计划报警\n"
                            + "发布计划ID: " + planId + "\n"
                            + "发布计划名称: " + planName + "\n"
                            + "开关关闭账号: " + JSONObject.toJSONString(accountNames) + "\n"
                            + "<at user_id=\"all\">所有人</at> ");
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("checkColdCrawlerPlan")
    public ReturnT<String> checkColdCrawlerPlan(String param) {
        long timeStamp = DateUtils.getBeforeDayStart(1);
        if (StringUtils.hasText(param)) {
            timeStamp = DateUtils.getStartOfDay(param, "yyyyMMdd");
        }
        String dateStr = DateUtils.timestampToYMDStr(timeStamp, "yyyyMMdd");
        timeStamp = timeStamp * 1000;
        List<String> planTags = Arrays.asList("账号联想_v1", "文章联想_v1", "品类冷启动");
        List<CrawlerPlan> crawlerPlanList = aigcBaseMapper.getColdCrawlerPlan(timeStamp, timeStamp + 86400000, planTags);
        if (CollectionUtil.isNotEmpty(crawlerPlanList)) {
            for (CrawlerPlan crawlerPlan : crawlerPlanList) {
                JSONObject crawlerModeValue = JSONObject.parseObject(crawlerPlan.getCrawlerModeValue());
                if (crawlerModeValue == null) {
                    continue;
                }
                JSONObject sourceModeValues = crawlerModeValue.getJSONObject("sourceModeValues");
                if (sourceModeValues == null) {
                    continue;
                }
                JSONArray inputModeValues = sourceModeValues.getJSONArray("inputModeValues");
                if (inputModeValues == null) {
                    continue;
                }
                log.info("checkColdCrawlerPlan crawlerPlan: {}", JSONObject.toJSONString(crawlerPlan));
                FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.DAILY.getRobotId(),
                        "【" + dateStr + "冷启计划抓取数量】\n"
                                + "计划ID: " + crawlerPlan.getId() + "\n"
                                + "计划名称: " + crawlerPlan.getName() + "\n"
                                + "计划标签: " + crawlerPlan.getPlanTag() + "\n"
                                + "计划抓取数量: " + inputModeValues.size() + "\n"
                                + "实际抓取数量: " + crawlerPlan.getCrawlerTotalNum() + "\n"
                                + "<at user_id=\"all\">所有人</at> ");
            }
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("checkColdProducePlanAudit")
    public ReturnT<String> checkColdProducePlanAudit(String param) {
        long timeStamp = DateUtils.getBeforeDayStart(1);
        if (StringUtils.hasText(param)) {
            timeStamp = DateUtils.getStartOfDay(param, "yyyyMMdd");
        }
        timeStamp = timeStamp * 1000;
        List<String> planIds = Arrays.asList("20241030070010871546586", "20240802080355355308981", "20240802021606053813696");
        List<ProducePlan> producePlans = producePlanRepository.findByIdIn(planIds);
        Map<String, String> producePlanMap = producePlans.stream().collect(Collectors.toMap(ProducePlan::getId, ProducePlan::getName));
        List<ProducePlanAuditCheckDTO> producePlanList = aigcBaseMapper.getProducePlanAudit(
                timeStamp, timeStamp + 86400000, planIds);
        Map<String, List<ProducePlanAuditCheckDTO>> producePlanAuditMap = producePlanList.stream()
                .collect(Collectors.groupingBy(ProducePlanAuditCheckDTO::getPlanId));
        for (String planId : planIds) {
            List<ProducePlanAuditCheckDTO> auditCheckDTOList = producePlanAuditMap.get(planId);
            Integer yesterdayAuditNum = 0, yesterdayAuditPassNum = 0, waitingAuditNum = 0;
            if (CollectionUtils.isNotEmpty(auditCheckDTOList)) {
                for (ProducePlanAuditCheckDTO producePlanAuditCheckDTO : auditCheckDTOList) {
                    if (Objects.equals(producePlanAuditCheckDTO.getAuditStatus(), ProduceContentAuditStatusEnum.waiting.getVal())) {
                        waitingAuditNum = producePlanAuditCheckDTO.getNum();
                    } else if (Objects.equals(producePlanAuditCheckDTO.getAuditStatus(), ProduceContentAuditStatusEnum.pass.getVal())) {
                        yesterdayAuditPassNum = producePlanAuditCheckDTO.getNum();
                        yesterdayAuditNum += producePlanAuditCheckDTO.getNum();
                    } else if (Objects.equals(producePlanAuditCheckDTO.getAuditStatus(), ProduceContentAuditStatusEnum.reject.getVal())) {
                        yesterdayAuditNum += producePlanAuditCheckDTO.getNum();
                    }
                }
            }
            log.info("checkColdProducePlanAudit planId: {}, planName: {}, yesterdayAuditNum: {}, " +
                            "yesterdayAuditPassNum: {}, waitingAuditNum: {}",
                    planId, producePlanMap.get(planId), yesterdayAuditNum, yesterdayAuditPassNum, waitingAuditNum);
            FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.DAILY.getRobotId(),
                    "【冷启生成计划审核数量】\n"
                            + "planId: " + planId + "\n"
                            + "planName: " + producePlanMap.get(planId) + "\n"
                            + "前一天审核数量: " + yesterdayAuditNum + "\n"
                            + "前一天审核通过数量: " + yesterdayAuditPassNum + "\n"
                            + "当前待审核数量: " + waitingAuditNum);
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("checkProducePlanAudit")
    public ReturnT<String> checkProducePlanAudit(String param) {
        long timeStamp = DateUtils.getBeforeDayStart(1);
        if (StringUtils.hasText(param)) {
            timeStamp = DateUtils.getStartOfDay(param, "yyyyMMdd");
        }
        timeStamp = timeStamp * 1000;
        List<String> planIds = Arrays.asList("20240802143345289374071", "20240802171417146947657",
                "20240804003153130851174", "20241030070010871546586", "20240802080355355308981",
                "20240802021606053813696");
        List<ProducePlan> producePlans = producePlanRepository.findByIdIn(planIds);
        Map<String, String> producePlanMap = producePlans.stream().collect(Collectors.toMap(ProducePlan::getId, ProducePlan::getName));
        List<ProducePlanAuditCheckDTO> producePlanList = aigcBaseMapper.getProducePlanAudit(
                timeStamp, timeStamp + 86400000, planIds);
        Map<String, List<ProducePlanAuditCheckDTO>> producePlanAuditMap = producePlanList.stream()
                .collect(Collectors.groupingBy(ProducePlanAuditCheckDTO::getPlanId));
        JSONArray jsonArray = new JSONArray();
        Integer totalAuditNum = 0, totalAuditPassNum = 0, totalWaitingAuditNum = 0;
        for (String planId : planIds) {
            JSONObject jsonObject = new JSONObject();
            List<ProducePlanAuditCheckDTO> auditCheckDTOList = producePlanAuditMap.get(planId);
            Integer yesterdayAuditNum = 0, yesterdayAuditPassNum = 0, waitingAuditNum = 0;
            if (CollectionUtils.isNotEmpty(auditCheckDTOList)) {
                for (ProducePlanAuditCheckDTO producePlanAuditCheckDTO : auditCheckDTOList) {
                    if (Objects.equals(producePlanAuditCheckDTO.getAuditStatus(), ProduceContentAuditStatusEnum.waiting.getVal())) {
                        waitingAuditNum = producePlanAuditCheckDTO.getNum();
                        totalWaitingAuditNum += waitingAuditNum;
                    } else if (Objects.equals(producePlanAuditCheckDTO.getAuditStatus(), ProduceContentAuditStatusEnum.pass.getVal())) {
                        yesterdayAuditPassNum = producePlanAuditCheckDTO.getNum();
                        totalAuditPassNum += yesterdayAuditPassNum;
                        yesterdayAuditNum += producePlanAuditCheckDTO.getNum();
                        totalAuditNum += yesterdayAuditNum;
                    } else if (Objects.equals(producePlanAuditCheckDTO.getAuditStatus(), ProduceContentAuditStatusEnum.reject.getVal())) {
                        yesterdayAuditNum += producePlanAuditCheckDTO.getNum();
                        totalAuditNum += yesterdayAuditNum;
                    }
                }
            }
            jsonObject.put("planId", planId);
            jsonObject.put("planName", producePlanMap.get(planId));
            jsonObject.put("yesterdayAuditNum", yesterdayAuditNum);
            jsonObject.put("yesterdayAuditPassNum", yesterdayAuditPassNum);
            jsonArray.add(jsonObject);
        }
        String alertMsg = "【长文生成计划审核量通知】\n";
        for (Object o : jsonArray) {
            JSONObject jsonObject = JSONObject.parseObject(o.toString());
            alertMsg += "计划名: " + jsonObject.getString("planName") + "\n";
            alertMsg += "昨日审核数量: " + jsonObject.getInteger("yesterdayAuditNum") +
                    "，昨日审核通过数量: " + jsonObject.getInteger("yesterdayAuditPassNum") + "\n";
        }
        alertMsg += "------------------------------------\n";
        alertMsg += "昨日总审核数量: " + totalAuditNum + "，总审核通过数量: " + totalAuditPassNum;
        log.info("checkProducePlanAudit alertMsg: {}", alertMsg);
        FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.ARTICLE_SAFE_AUDIT.getRobotId(), alertMsg);
        return ReturnT.SUCCESS;
    }

    @XxlJob("migrateCrawlerRootSourceId")
    public ReturnT<String> migrateCrawlerRootSourceId(String param) {
        try {
            long timeStamp = DateUtils.getBeforeDayStart(1);
            if (StringUtils.hasText(param)) {
                timeStamp = DateUtils.getStartOfDay(param, "yyyyMMdd");
            }
            List<GetOffVideoCrawler> getOffVideoCrawlerList = getOffVideoCrawlerRepository.getByPublishTimeBetween(timeStamp, timeStamp + 86400);
            List<String> traceIds = getOffVideoCrawlerList.stream().map(GetOffVideoCrawler::getTraceId).distinct().collect(Collectors.toList());
            List<LongArticlesVideo> longArticlesVideoList = longArticlesVideoRepository.getByTraceIdIn(traceIds);
            CountDownLatch cdl = new CountDownLatch(longArticlesVideoList.size());
            for (LongArticlesVideo longArticlesVideo : longArticlesVideoList) {
                thread.submit(() -> {
                    try {
                        processCrawlerEachData(longArticlesVideo);
                    } finally {
                        cdl.countDown();
                    }
                });
            }
            cdl.await();
        } catch (Exception e) {
            log.error("migrateCrawlerRootSourceId exception: {}", e.getMessage(), e);
            return ReturnT.FAIL;
        }
        return ReturnT.SUCCESS;
    }

    /**
     * Processes each data tuple and updates the rootSourceId.
     */
    public void processCrawlerEachData(LongArticlesVideo longArticlesVideo) {
        try {
            String[] results = {longArticlesVideo.getResult1(), longArticlesVideo.getResult2(), longArticlesVideo.getResult3()};
            List<LongArticlesRootSourceId> saveList = new ArrayList<>();
            for (String result : results) {
                if (result != null && !result.isEmpty()) {
                    try {
                        JSONObject jsonNode = JSONObject.parseObject(result);
                        String productionPath = jsonNode.getString("productionPath");
                        String rootSourceId = productionPath.split("rootSourceId%3D")[1];
                        String videoId = productionPath.split("videos%3Fid%3D")[1].split("%26su%")[0];
                        LongArticlesRootSourceId saveItem = new LongArticlesRootSourceId();
                        saveItem.setRootSourceId(rootSourceId);
                        saveItem.setAccountName(longArticlesVideo.getAccountName());
                        saveItem.setGhId(longArticlesVideo.getGhId());
                        saveItem.setArticleTitle(jsonNode.getString("productionName"));
                        saveItem.setRequestTime(longArticlesVideo.getRequestTimeStamp());
                        saveItem.setTraceId(longArticlesVideo.getTraceId());
                        saveItem.setPushType(2);
                        saveItem.setVideoId(Long.valueOf(videoId));
                        saveList.add(saveItem);
                    } catch (Exception e) {
                        log.error("processCrawlerEachData LongArticlesRootSourceId saveError: {}", result, e);
                    }
                }
            }
            saveLongArticlesRootSourceId(longArticlesVideo.getTraceId(), saveList);
        } catch (DuplicateKeyException e) {
            log.error("Error processCrawlerEachData: {}", JSONObject.toJSONString(longArticlesVideo), e);
            String errMsg = e.getMessage().substring(0, e.getMessage().indexOf("### The error"));
            sendFeishuProcessCrawlerEachData("processCrawlerEachData", longArticlesVideo.getTraceId(), errMsg);
        } catch (Exception e) {
            log.error("Error processCrawlerEachData: {}", JSONObject.toJSONString(longArticlesVideo), e);
            sendFeishuProcessCrawlerEachData("processCrawlerEachData", longArticlesVideo.getTraceId(), e.getMessage());
        }
    }

    private void saveLongArticlesRootSourceId(String traceId, List<LongArticlesRootSourceId> saveList) {
        if (CollectionUtil.isNotEmpty(saveList)) {
            int count = longArticlesRootSourceIdRepository.countByTraceId(traceId);
            if (count > 0) {
                return;
            }
            saveList = saveList.stream().filter(item -> StringUtils.hasText(item.getRootSourceId())).collect(Collectors.toList());
            List<String> rootSourceIds = saveList.stream().map(LongArticlesRootSourceId::getRootSourceId).distinct().collect(Collectors.toList());
            List<LongArticlesRootSourceId> existList = longArticlesRootSourceIdRepository.getByRootSourceIdIn(rootSourceIds);
            for (LongArticlesRootSourceId existItem : existList) {
                saveList.removeIf(item -> item.getRootSourceId().equals(existItem.getRootSourceId()));
            }
            if (CollectionUtil.isNotEmpty(saveList)) {
                longArticleBaseMapper.batchInsertLongArticlesRootSourceId(saveList);
            }
        }
    }

    private void sendFeishuProcessCrawlerEachData(String method, String traceId, String errMsg) {
        FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.JOB.getRobotId(),
                method + " 异常\n"
                        + "异常数据TraceId: " + traceId + "\n"
                        + "错误信息：" + errMsg);
    }

    @XxlJob("migrateArticleRootSourceId")
    public ReturnT<String> migrateArticleRootSourceId(String param) {
        try {
            long timeStamp = DateUtils.getBeforeDayStart(1);
            if (StringUtils.hasText(param)) {
                timeStamp = DateUtils.getStartOfDay(param, "yyyyMMdd");
            }
            List<GetOffVideoArticle> getOffVideoArticleList = getOffVideoArticleRepository.getByPublishTimeBetween(timeStamp, timeStamp + 86400);
            List<String> traceIds = getOffVideoArticleList.stream().map(GetOffVideoArticle::getTraceId).distinct().collect(Collectors.toList());
            List<LongArticlesMatchVideo> longArticlesMatchVideoList = longArticlesMatchVideoRepository.getByTraceIdIn(traceIds);
            CountDownLatch cdl = new CountDownLatch(longArticlesMatchVideoList.size());
            for (LongArticlesMatchVideo longArticlesMatchVideo : longArticlesMatchVideoList) {
                thread.submit(() -> {
                    try {
                        processArticleEachData(longArticlesMatchVideo);
                    } finally {
                        cdl.countDown();
                    }
                });
            }
            cdl.await();
        } catch (Exception e) {
            log.error("migrateArticleRootSourceId exception: {}", e.getMessage(), e);
            return ReturnT.FAIL;
        }
        return ReturnT.SUCCESS;
    }

    /**
     * Processes each data tuple and updates the rootSourceId.
     */
    public void processArticleEachData(LongArticlesMatchVideo longArticlesMatchVideo) {
        try {
            List<String> results = JSONArray.parseArray(longArticlesMatchVideo.getResponse(), String.class);
            List<LongArticlesRootSourceId> saveList = new ArrayList<>();
            for (String result : results) {
                JSONObject jsonNode = JSONObject.parseObject(result);
                LongArticlesRootSourceId saveItem = new LongArticlesRootSourceId();
                saveItem.setRootSourceId(jsonNode.getString("rootSourceId"));
                saveItem.setAccountName(longArticlesMatchVideo.getAccountName());
                saveItem.setGhId(longArticlesMatchVideo.getGhId());
                saveItem.setRequestTime(longArticlesMatchVideo.getRequestTimestamp());
                saveItem.setTraceId(longArticlesMatchVideo.getTraceId());
                saveItem.setPushType(2);
                saveItem.setVideoId(jsonNode.getLong("videoId"));
                saveList.add(saveItem);
            }
            saveLongArticlesRootSourceId(longArticlesMatchVideo.getTraceId(), saveList);
        } catch (DuplicateKeyException e) {
            log.error("Error processArticleEachData: {}", JSONObject.toJSONString(longArticlesMatchVideo), e);
            String errMsg = e.getMessage().substring(0, e.getMessage().indexOf("### The error"));
            sendFeishuProcessCrawlerEachData("processArticleEachData", longArticlesMatchVideo.getTraceId(), errMsg);
        } catch (Exception e) {
            log.error("Error processArticleEachData: {}", JSONObject.toJSONString(longArticlesMatchVideo), e);
            sendFeishuProcessCrawlerEachData("processArticleEachData", longArticlesMatchVideo.getTraceId(), e.getMessage());
        }
    }

    @XxlJob("accountAvgInfoV3Job")
    public ReturnT<String> accountAvgInfoV3Job(String param) {
        long timeStamp = DateUtils.getBeforeDayStart(1);
        if (StringUtils.hasText(param)) {
            timeStamp = DateUtils.getStartOfDay(param, "yyyyMMdd");
        }
        String dateStr = DateUtils.timestampToYMDStr(timeStamp, "yyyy-MM-dd");
        List<AccountTypeFansDTO> accountList = aigcBaseMapper.getAccountTypeFans();
        List<LongArticlesReadRate> readRateList = longArticlesReadRateRepository.findAll();
        Map<String, Double> rateDict = readRateList.stream().collect(Collectors.toMap(o -> o.getGhId() + "_" + o.getPosition(),
                LongArticlesReadRate::getReadRateAvg));
        for (AccountTypeFansDTO accountTypeFansDTO : accountList) {
            saveAccountAvgInfo(accountTypeFansDTO, rateDict, dateStr);
        }
        return ReturnT.SUCCESS;
    }

    public void saveAccountAvgInfo(AccountTypeFansDTO account, Map<String, Double> rateDict, String date) {
        int businessType = touliuAccountGhIds.contains(account.getGhId()) ?
                AccountBusinessTypeEnum.TOULIU.getType() : AccountBusinessTypeEnum.CHANGWEN.getType();
        if (account.getFollowerCount() > 0) {
            List<AccountAvgInfo> saveList = new ArrayList<>();
            for (int index = 1; index <= 8; index++) {
                String ghIdPosition = account.getGhId() + "_" + index;
                if (rateDict.containsKey(ghIdPosition)) {
                    double rate = rateDict.get(ghIdPosition);
                    double readAvg = account.getFollowerCount() * rate;
                    AccountAvgInfo accountAvgInfo = new AccountAvgInfo();
                    accountAvgInfo.setGhId(account.getGhId());
                    accountAvgInfo.setPosition(String.valueOf(index));
                    accountAvgInfo.setUpdateTime(date);
                    accountAvgInfo.setAccountName(account.getName());
                    accountAvgInfo.setFans(account.getFollowerCount());
                    accountAvgInfo.setReadAvg(readAvg);
                    accountAvgInfo.setStatus(1);
                    accountAvgInfo.setAccountType(account.getAccountType());
                    accountAvgInfo.setAccountMode(account.getModeType());
                    accountAvgInfo.setAccountSource(account.getAccountSourceName());
                    accountAvgInfo.setAccountStatus(account.getStatus());
                    accountAvgInfo.setBusinessType(businessType);
                    accountAvgInfo.setReadRateAvg(rate);
                    saveList.add(accountAvgInfo);
                }
            }
            if (CollectionUtil.isNotEmpty(saveList)) {
                try {
                    crawlerBaseMapper.batchInsertAccountAvgInfo(saveList);
                } catch (Exception e) {
                    crawlerBaseMapper.deleteAccountAvgInfoByGhIdAndUpdateTime(account.getGhId(), date);
                    crawlerBaseMapper.batchInsertAccountAvgInfo(saveList);
                }
                crawlerBaseMapper.updateAccountAvgInfoStatus(account.getGhId(), date);
            }
        }
    }

    @XxlJob("alertPushMessageJob")
    public ReturnT<String> alertPushMessageJob(String param) {
        long endTimestamp = System.currentTimeMillis();
        long startTimestamp = endTimestamp - MILLISECOND_HOUR;
        long ytdStartTimestamp = startTimestamp - MILLISECOND_DAY;
        long ytdEndTimestamp = endTimestamp - MILLISECOND_DAY;
        PushMessageCallbackExample example = new PushMessageCallbackExample();
        example.createCriteria().andCreateTimeBetween(new Date(startTimestamp), new Date(endTimestamp));
        long nowCount = newPushMessageCallbackMapper.countByExample(example);
        if (nowCount == 0) {
            LarkRobotUtil.sendMessage("最近一小时回调消息为0");
        }
        PushMessageCallbackExample ytdExample = new PushMessageCallbackExample();
        ytdExample.createCriteria().andCreateTimeBetween(new Date(ytdStartTimestamp), new Date(ytdEndTimestamp));
        long ydtCount = newPushMessageCallbackMapper.countByExample(example);
        if (nowCount + 1000 < ydtCount) {
            if (ydtCount == 0) {
                ydtCount = 1L;
            }
            double res = ((double) nowCount - ydtCount) * 100 / ydtCount;
            LarkRobotUtil.sendMessage("最近一小时回调消息量：" + nowCount + "\n"
                    + "昨日消息回调量：" + ydtCount + "\n"
                    + "百分比为：" + String.format("%.2f", res) + "%");
        }
        return ReturnT.SUCCESS;
    }

    @XxlJob("delPushMessageCallbackJob")
    public ReturnT<String> delPushMessageCallbackJob(String param) {
        Calendar calendar = Calendar.getInstance();
        Long beforeDayStart = DateUtils.getBeforeDayStart(7);
        Date startDay = new Date(beforeDayStart * 1000);
        if (StringUtils.hasText(param)) {
            startDay = DateUtils.getDate(param);
        }
        for (int i = 0; i < 30; i++) {
            // 计算当前遍历日期
            Date currentStartDate = (Date) startDay.clone();
            calendar.setTime(currentStartDate);
            calendar.add(Calendar.DAY_OF_MONTH, -i); // 向前推i天

            // 生成开始时间和结束时间
            Date startTime = calendar.getTime(); // 开始时间
            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            Date endTime = calendar.getTime(); // 结束时间

            PushMessageCallbackExample example = new PushMessageCallbackExample();
            example.createCriteria().andCreateTimeBetween(startTime, endTime);
            long mysqlCount = newPushMessageCallbackMapper.countByExample(example);
            if (mysqlCount == 0) {
                break;
            }
            String pt = DateUtils.getDateString(startTime.getTime());
            String sql = String.format("SELECT count(*) FROM push_message_callback WHERE pt = %s;", pt);
            List<Record> recordList = odpsManager.query(sql);
            if (CollectionUtil.isEmpty(recordList)) {
                LarkRobotUtil.sendMessage("查询hive失败" + pt);
                return ReturnT.FAIL;
            }
            Long hiveCount = recordList.get(0).getBigint(0);
            if (Math.abs(mysqlCount - hiveCount) > 0) {
                LarkRobotUtil.sendMessage("数量异常" + pt + "\n mysql数量：" + mysqlCount + "\n hive数量：" + hiveCount);
                return ReturnT.FAIL;
            }
            newPushMessageCallbackMapper.deleteByExample(example);
        }
        return ReturnT.SUCCESS;
    }


    @XxlJob("syncArticleRootPublishId")
    public ReturnT<String> syncArticleRootPublishId(String param) {
        ArticleFindSourceParam findSourceParam = new ArticleFindSourceParam();
        findSourceParam.setDateStr(param);
        articleService.findSource(findSourceParam);
        return ReturnT.SUCCESS;
    }

    @XxlJob("articlePromotionTraceability")
    public ReturnT<String> articlePromotionTraceability(String channelContentId) {
        articleService.articlePromotionTraceability(channelContentId);
        return ReturnT.SUCCESS;
    }

    @XxlJob("articleCategoryJob")
    public ReturnT<String> articleCategoryJob(String param) {
        articleCategoryService.articleCategory();
        return ReturnT.SUCCESS;
    }

    @XxlJob("articleCategoryJobRetry")
    public ReturnT<String> articleCategoryJobRetry(String param) {
        articleCategoryService.articleCategoryJobRetry();
        return ReturnT.SUCCESS;
    }

    @XxlJob("articlePromotion")
    public ReturnT<String> articlePromotion(String param) {
        articlePromotionService.articlePromotion("【1】", "direct", "头条利用池_v1",
                "头条利用池_v4", 500, 1.1, Lists.newArrayList(1));
        articlePromotionService.articlePromotion("【1】", "direct", "头条利用池_v2",
                "头条利用池_v4", 250, 1.21, Lists.newArrayList(2));
        articlePromotionService.articlePromotion("【2】", "direct", "次条利用池_v1",
                "次条利用池_v4", 100, 1.33, Lists.newArrayList(3, 4, 5, 6, 7, 8));
        return ReturnT.SUCCESS;
    }

    @XxlJob("articleVideoPoolPromotion")
    public ReturnT<String> articleVideoPoolPromotion(String param) {
        articlePromotionService.articleVideoPoolPromotion(500, 1.1,
                Lists.newArrayList(1), ContentPoolEnum.autoArticlePoolLevel1.getValue());
        articlePromotionService.articleVideoPoolPromotion(250, 1.21,
                Lists.newArrayList(2), ContentPoolEnum.autoArticlePoolLevel1.getValue());
        articlePromotionService.articleVideoPoolPromotion(100, 1.33,
                Lists.newArrayList(3, 4, 5, 6, 7, 8), ContentPoolEnum.autoArticlePoolLevel3.getValue());
        return ReturnT.SUCCESS;
    }

    @XxlJob("refreshArticleHisCache")
    public ReturnT<String> refreshArticleHisCache(String param) {
        // 刷新历史表现缓存
        long count = articleTitleHisCacheRepository.count();
        int pageSize = 100;
        long page = (count / pageSize) + 1;
        for (int i = 0; i < page; i++) {
            Page<ArticleTitleHisCache> articleTitleHisCachePage = articleTitleHisCacheRepository.findAll(
                    PageRequest.of(i, pageSize));
            List<ArticleTitleHisCache> cacheList = articleTitleHisCachePage.getContent();
            if (CollectionUtils.isEmpty(cacheList)) {
                continue;
            }
            Map<String, Map<String, ArticleTitleHisCache>> cacheMap = cacheList.stream().collect(
                    Collectors.groupingBy(ArticleTitleHisCache::getType,
                            Collectors.toMap(ArticleTitleHisCache::getSourceId, Function.identity())));
            for (Map.Entry<String, Map<String, ArticleTitleHisCache>> typeEntry : cacheMap.entrySet()) {
                String type = typeEntry.getKey();
                Map<String, ArticleTitleHisCache> sourceIdToCacheMap = typeEntry.getValue();
                Set<String> sourceIdList = sourceIdToCacheMap.keySet();
                List<TitleHisCacheParam> paramList = sourceIdList.stream().map(sourceId -> {
                    ArticleTitleHisCache cache = cacheMap.get(type).get(sourceId);
                    TitleHisCacheParam cacheParam = new TitleHisCacheParam();
                    cacheParam.setSourceId(sourceId);
                    cacheParam.setTitleMd5(cache.getTitleMd5());
                    cacheParam.setTitle(cache.getTitle());
                    cacheParam.setCrawlerTitle(cache.getCrawlerTitle());
                    cacheParam.setCrawlerChannelContentId(cache.getChannelContentId());
                    if (StringUtils.hasText(cache.getCategory())) {
                        cacheParam.setCategory(JSONArray.parseArray(cache.getCategory(), String.class));
                    }
                    return cacheParam;
                }).collect(Collectors.toList());
                Map<String, Content> articlesWithHistory = recallService.getArticleWithHistory(paramList, type);
                for (String sourceId : sourceIdList) {
                    Content content = articlesWithHistory.get(sourceId);
                    if (Objects.nonNull(content) && CollectionUtils.isNotEmpty(content.getHisPublishArticleList())) {
                        ArticleTitleHisCache cache = sourceIdToCacheMap.get(sourceId);
                        cache.setHisPublishArticleList(JSONObject.toJSONString(content.getHisPublishArticleList()));
                        if (CollectionUtil.isNotEmpty(content.getCategory())) {
                            cache.setCategory(JSONObject.toJSONString(content.getCategory()));
                        }
                        cache.setUpdateTimestamp(System.currentTimeMillis());
                        articleTitleHisCacheRepository.save(cache);
                    }
                }
            }
        }

        return ReturnT.SUCCESS;
    }

}
