package com.tzld.longarticle.recommend.server.service.recommend;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.tzld.longarticle.recommend.server.common.enums.FieshuTableColumnDataTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishContentStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishPlanInputSourceTypesEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PushTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.cgi.PQVideoAuditResultEnum;
import com.tzld.longarticle.recommend.server.common.enums.cgi.PQVideoSensitiveLevelEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleDeleteStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleMatchContentStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.FeishuRobotIdEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.cgi.PQVideoAuditResult;
import com.tzld.longarticle.recommend.server.model.dto.*;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishAccount;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishContent;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.*;
import com.tzld.longarticle.recommend.server.model.param.ArticleDangerFindDeleteParam;
import com.tzld.longarticle.recommend.server.model.param.PublishContentParam;
import com.tzld.longarticle.recommend.server.model.vo.FeishuTableDTO;
import com.tzld.longarticle.recommend.server.remote.WxAccessTokenRemoteService;
import com.tzld.longarticle.recommend.server.remote.WxArticleDeleteService;
import com.tzld.longarticle.recommend.server.remote.pq.PQVideoAuditResultService;
import com.tzld.longarticle.recommend.server.remote.pq.PQVideoAuditStartProcessService;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishAccountRepository;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishContentRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.*;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.Md5Util;
import com.tzld.longarticle.recommend.server.util.feishu.FeishuMessageSender;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.net.URL;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ArticleAuditService {

    @Autowired
    private GetOffVideoArticleRepository getOffVideoArticleRepository;
    @Autowired
    private LongArticlesMatchVideoRepository longArticlesMatchVideoRepository;
    @Autowired
    private LongArticleCrawlerVideoRepository longArticleCrawlerVideoRepository;
    @Autowired
    private LongArticleAuditDeleteRepository longArticleAuditDeleteRepository;
    @Autowired
    private LongArticleVideoAuditRepository longArticleVideoAuditRepository;
    @Autowired
    private WxArticleDeleteService wxArticleDeleteService;
    @Autowired
    private WxAccessTokenRemoteService wxAccessTokenRemoteService;
    @Autowired
    private AigcBaseMapper aigcBaseMapper;
    @Autowired
    private PublishAccountRepository publishAccountRepository;
    @Autowired
    private ArticleRepository articleRepository;
    @Autowired
    private ArticleUnsafeTitleRepository articleUnsafeTitleRepository;
    @Autowired
    private PQVideoAuditStartProcessService pqVideoAuditStartProcessService;
    @Autowired
    private PQVideoAuditResultService pqVideoAuditResultService;
    @Autowired
    private LongArticleBaseMapper longArticleBaseMapper;
    @Autowired
    private PublishContentRepository publishContentRepository;
    @Autowired
    private LongArticlesTextRepository longArticlesTextRepository;


    @XxlJob("articleVideoAudit")
    public ReturnT<String> articleVideoAudit(String param) {
        long publishTime = DateUtils.getTodayStart() / 1000;
        if (StringUtils.hasText(param)) {
            publishTime = DateUtils.getStartOfDay(param, "yyyyMMdd");
        }
        List<GetOffVideoArticle> getOffVideoArticleList = getOffVideoArticleRepository.getByPublishTimeGreaterThanEqual(publishTime);
        List<String> traceIds = getOffVideoArticleList.stream()
                .filter(o -> !o.getTraceId().startsWith("direct"))
                .map(GetOffVideoArticle::getTraceId).distinct().collect(Collectors.toList());
        if (CollectionUtil.isEmpty(traceIds)) {
            return ReturnT.SUCCESS;
        }
        List<LongArticlesMatchVideo> longArticlesMatchVideoList = longArticlesMatchVideoRepository.getByTraceIdIn(traceIds);
        Map<String, LongArticlesMatchVideo> longarticlesMatchVideoMap = longArticlesMatchVideoList.stream()
                .collect(Collectors.toMap(LongArticlesMatchVideo::getTraceId, Function.identity()));
        List<String> existsOssPath = longArticleBaseMapper.getExistsOssPath();
        for (String traceId : traceIds) {
            try {
                LongArticlesMatchVideo longArticlesMatchVideo = longarticlesMatchVideoMap.get(traceId);
                List<LongArticlesMatchVideoResponse> responseList = JSONArray.parseArray(longArticlesMatchVideo.getResponse()
                        , LongArticlesMatchVideoResponse.class);
                for (LongArticlesMatchVideoResponse response : responseList) {
                    String ossPath = response.getVideoOSS();
                    if (StringUtils.hasText(ossPath)) {
                        if (existsOssPath.contains(ossPath)) {
                            continue;
                        }
                        existsOssPath.add(ossPath);
                    }
                    LongArticleVideoAudit videoAudit = new LongArticleVideoAudit();
                    videoAudit.setVideoId(response.getVideoID());
                    videoAudit.setTraceId(traceId);
                    videoAudit.setContentId(longArticlesMatchVideo.getContentId());
                    videoAudit.setOssPath(ossPath);
                    videoAudit.setStatus(StatusEnum.ZERO.getCode());
                    videoAudit.setCreateTimestamp(System.currentTimeMillis());
                    longArticleVideoAuditRepository.save(videoAudit);
                    // 调用PQ 审核视频
                    try {
                        Long taskId = pqVideoAuditStartProcessService.startProcess(response.getVideoID());
                        if (Objects.nonNull(taskId)) {
                            videoAudit.setTaskId(taskId);
                            longArticleVideoAuditRepository.save(videoAudit);
                        } else {
                            log.error("PQVideoAuditStartProcess start process error videoId:{} ", response.getVideoID());
                        }
                    } catch (Exception e) {
                        log.error("PQVideoAuditStartProcess start process videoId:{} error:{}", response.getVideoID(), e.getMessage());
                    }
                }
            } catch (Exception e) {
                log.error("articleVideoAudit traceId:{} error:{}", traceId, e.getMessage());
            }
        }

        return ReturnT.SUCCESS;
    }

    private String getOssPath(String ossUrl) {
        String result = "";
        if (ossUrl == null || ossUrl.isEmpty()) {
            return result;
        }
        try {
            URL url = new URL(ossUrl);
            // 去除域名
            result = url.getPath();
            // 获取后缀.位置
            int endIndex = result.lastIndexOf(".");
            if (endIndex == -1) {
                endIndex = result.length();
            }
            // 去除 / 及 文件后缀
            result = result.substring(1, endIndex);
        } catch (Exception e) {
            log.error("getOssPath error:{}", e.getMessage());
        }
        return result;
    }

    @XxlJob("articleVideoAuditResult")
    public ReturnT<String> articleVideoAuditResult(String param) {
        List<LongArticleVideoAudit> list = longArticleVideoAuditRepository.getByStatusAndTaskIdIsNotNull(StatusEnum.ZERO.getCode());
        for (List<LongArticleVideoAudit> partition : Lists.partition(list, 10)) {
            List<Long> taskIds = partition.stream().map(LongArticleVideoAudit::getTaskId).collect(Collectors.toList());
            Map<Long, LongArticleVideoAudit> map = partition.stream().collect(Collectors.toMap(LongArticleVideoAudit::getTaskId, Function.identity()));
            List<PQVideoAuditResult> resultList = pqVideoAuditResultService.getResult(taskIds);
            resultList.forEach(result -> saveVideoAuditResult(map.get(result.getTaskId()), result));
        }
        return ReturnT.SUCCESS;
    }


    public void saveVideoAuditResult(LongArticleVideoAudit longArticleVideoAudit, PQVideoAuditResult result) {

        if (Objects.equals(result.getAuditResult(), PQVideoAuditResultEnum.PASS.getStatus())) {
            // 审核通过，更新文章状态
            longArticleVideoAudit.setStatus(PQVideoAuditResultEnum.PASS.getStatus());
            longArticleVideoAudit.setFinishTimestamp(System.currentTimeMillis());
            longArticleVideoAuditRepository.save(longArticleVideoAudit);
        } else if (Objects.equals(result.getAuditResult(), PQVideoAuditResultEnum.REJECT.getStatus())) {
            // 审核不通过，删除文章
            PQVideoSensitiveLevelEnum sensitiveLevelEnum = PQVideoSensitiveLevelEnum.from(result.getSensitiveLevel());
            if (sensitiveLevelEnum.getLevel() >= PQVideoSensitiveLevelEnum.MID_SENSITIVE.getLevel()) {
                longArticleVideoAudit.setStatus(PQVideoAuditResultEnum.REJECT.getStatus());
            } else {
                longArticleVideoAudit.setStatus(PQVideoAuditResultEnum.PASS.getStatus());
            }
            longArticleVideoAudit.setFailReason(sensitiveLevelEnum.getDescription());
            longArticleVideoAudit.setFinishTimestamp(System.currentTimeMillis());
            longArticleVideoAuditRepository.save(longArticleVideoAudit);
            // 构建删除文章记录 并保存
            // saveDeleteRecord(longArticleVideoAudit.getOssPath());
            // 暂时不做删除 先发送通知
            if (longArticleVideoAudit.getStatus().equals(PQVideoAuditResultEnum.REJECT.getStatus())) {
                FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.ARTICLE_DELETE.getRobotId(),
                        "视频审核不通过【"+sensitiveLevelEnum.getDescription()+"】\n" +
                                "traceId：" + longArticleVideoAudit.getTraceId() + "\n" +
                                "视频id：" + longArticleVideoAudit.getVideoId() + "\n" +
                                "管理后台地址：https://admin.piaoquantv.com/cms/post-detail/" + longArticleVideoAudit.getVideoId() + "/detail\n" +
                                "操作删除视频及文章（慎点）：" +
                                "http://192.168.203.83:30081/articleAudit/saveDeleteRecord?videoId=" + longArticleVideoAudit.getVideoId());
            }
        }
    }

    public String saveDeleteRecord(Long videoId) {
        GetOffVideoArticle getOffVideos = getOffVideoArticleRepository.getByVideoId(videoId);
        String traceId = getOffVideos.getTraceId();
        LongArticlesMatchVideo matchVideo = longArticlesMatchVideoRepository.getByTraceId(traceId);
        List<LongArticlesMatchVideoResponse> responseList = JSONArray.parseArray(matchVideo.getResponse()
                , LongArticlesMatchVideoResponse.class);
        LongArticlesMatchVideoResponse response = responseList.stream()
                .filter(o -> o.getVideoID() == videoId).findFirst().orElse(null);
        if (Objects.isNull(response) || !StringUtils.hasText(response.getVideoOSS())) {
            return "matchVideo response videoOss null videoId:" + videoId + " traceId:" + traceId;
        }
        List<LongArticleCrawlerVideo> crawlerVideoList = longArticleCrawlerVideoRepository.getByVideoOssPath(response.getVideoOSS());
        if (CollectionUtils.isEmpty(crawlerVideoList)) {
            return "matchVideo response videoOss findCrawlerVideo Null videoId:" + videoId +
                    " traceId:" + traceId + " videoOss:" + response.getVideoOSS();
        }
        for (LongArticleCrawlerVideo longArticleCrawlerVideo : crawlerVideoList) {
            longArticleCrawlerVideo.setIsIllegal(1);
            longArticleCrawlerVideoRepository.save(longArticleCrawlerVideo);
        }
        List<String> contentIds = crawlerVideoList.stream().map(LongArticleCrawlerVideo::getContentId).collect(Collectors.toList());
        List<LongArticlesMatchVideo> matchVideoList = longArticlesMatchVideoRepository.getByContentIdIn(contentIds);
        Map<String, LongArticlesMatchVideo> matchVideoMap = matchVideoList.stream()
                .collect(Collectors.toMap(LongArticlesMatchVideo::getTraceId, Function.identity()));
        List<String> traceIds = matchVideoList.stream().map(LongArticlesMatchVideo::getTraceId).collect(Collectors.toList());
        List<PublishContentMiniprogramDTO> publishContentMiniprogramList = aigcBaseMapper.getPublishContentByTraceIdIn(traceIds);
        List<String> publishContentIds = publishContentMiniprogramList.stream()
                .map(PublishContentMiniprogramDTO::getPublishContentId).collect(Collectors.toList());
        Map<String, String> PublishTraceIdMap = publishContentMiniprogramList.stream()
                .collect(Collectors.toMap(PublishContentMiniprogramDTO::getPublishContentId, PublishContentMiniprogramDTO::getTraceId));
        // 过滤状态非已发布内容
        List<PublishContent> publishContentList = publishContentRepository.getByIdIn(publishContentIds);
        List<String> publishedIds = publishContentList.stream()
                .filter(o -> Objects.equals(o.getStatus(), PublishContentStatusEnum.published.getVal()))
                .map(PublishContent::getId).collect(Collectors.toList());
        publishContentIds = publishContentIds.stream().filter(publishedIds::contains).collect(Collectors.toList());
        List<String> deleteTraceIds = publishContentIds.stream().map(PublishTraceIdMap::get).collect(Collectors.toList());
        // 重新匹配小程序
        List<String> reMatchTraceIds = traceIds.stream().filter(o -> !deleteTraceIds.contains(o)).collect(Collectors.toList());
        buildReMatchRecord(reMatchTraceIds, response.getVideoOSS(), matchVideoMap);
        // 文章删除
        buildArticleAuditDelete(publishContentIds);
        return null;
    }

    private void buildReMatchRecord(List<String> reMatchTraceIds, String ossPath,
                                    Map<String, LongArticlesMatchVideo> matchVideoMap) {
        if (CollectionUtils.isEmpty(reMatchTraceIds)) {
            return;
        }
        Long now = System.currentTimeMillis() / 1000;
        List<ArticleReMatchRecord> reMatchRecordList = new ArrayList<>();
        for (String traceId : reMatchTraceIds) {
            LongArticlesMatchVideo matchVideo = matchVideoMap.get(traceId);
            if (Objects.isNull(matchVideo)
                    || ArticleMatchContentStatusEnum.ETL_COMPLETE_STATUS.getCode().equals(matchVideo.getContentStatus())
                    || ArticleMatchContentStatusEnum.MISMATCH_STATUS.getCode().equals(matchVideo.getContentStatus())
                    || ArticleMatchContentStatusEnum.EXIT_STATUS.getCode().equals(matchVideo.getContentStatus())
                    || ArticleMatchContentStatusEnum.FAIL_STATUS.getCode().equals(matchVideo.getContentStatus())) {
                continue;
            }
            ArticleReMatchRecord record = new ArticleReMatchRecord();
            record.setTraceId(traceId);
            record.setContentId(matchVideo.getContentId());
            record.setOssPath(ossPath);
            record.setStatus(0);
            if (StringUtils.hasText(matchVideo.getResponse())) {
                record.setOldResponse(matchVideo.getResponse());
            }
            record.setCreateTimestamp(now);
            reMatchRecordList.add(record);
        }
        longArticleBaseMapper.batchInsertArticleReMatchRecord(reMatchRecordList);
    }

    private void buildArticleAuditDelete(List<String> publishContentIds) {
        if (CollectionUtils.isEmpty(publishContentIds)) {
            return;
        }
        // 过滤已删除数据
        List<LongArticleAuditDelete> existList = longArticleAuditDeleteRepository.getByPublishContentIdIn(publishContentIds);
        List<String> existContentIds = existList.stream().map(LongArticleAuditDelete::getPublishContentId).collect(Collectors.toList());
        publishContentIds = publishContentIds.stream().filter(id -> !existContentIds.contains(id)).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(publishContentIds)) {
            return;
        }
        List<PublishGzhPushContentRelDTO> pushContentRelList = aigcBaseMapper.getPushContentRelByPublishContentIdIn(publishContentIds);
        if (CollectionUtils.isEmpty(pushContentRelList)) {
            return;
        }
        List<String> pushIds = pushContentRelList.stream().map(PublishGzhPushContentRelDTO::getPushId).collect(Collectors.toList());
        Map<String, String> publishPushIdMap = pushContentRelList.stream()
                .collect(Collectors.toMap(PublishGzhPushContentRelDTO::getPublishContentId,
                        PublishGzhPushContentRelDTO::getPushId,
                        (o1, o2) -> o2));
        List<PublishGzhPushDTO> pushList = aigcBaseMapper.getPushByPushIdIn(pushIds);
        Map<String, PublishGzhPushDTO> pushDTOMap = pushList.stream()
                .collect(Collectors.toMap(PublishGzhPushDTO::getPushId, Function.identity()));
        Map<String, String> pushIdMap = pushList.stream().filter(o -> StringUtils.hasText(o.getGroupPushMsgId()))
                .collect(Collectors.toMap(PublishGzhPushDTO::getPushId, PublishGzhPushDTO::getGroupPushMsgId));
        List<String> publishAccountIds = pushList.stream().map(PublishGzhPushDTO::getPublishAccountId).collect(Collectors.toList());
        Map<String, String> pushAccountMap = pushList.stream()
                .collect(Collectors.toMap(PublishGzhPushDTO::getPushId, PublishGzhPushDTO::getPublishAccountId));
        List<PublishAccount> publishAccountList = publishAccountRepository.getByIdIn(publishAccountIds);
        Map<String, String> publishAccountMap = publishAccountList.stream()
                .collect(Collectors.toMap(PublishAccount::getId, PublishAccount::getGhId));
        List<PublishGzhPushContentRelDTO> groupPushRelList = aigcBaseMapper.getGroupPushRelByPushIdIn(pushIds);
        Map<String, List<PublishGzhPushContentRelDTO>> groupPushRelMap = groupPushRelList.stream()
                .collect(Collectors.groupingBy(PublishGzhPushContentRelDTO::getPushId));
        List<PublishAccountTypeDTO> accountTypeList = aigcBaseMapper.getAccountTypeList();
        Map<String, PublishAccountTypeDTO> accountTypeMap = accountTypeList.stream()
                .collect(Collectors.toMap(PublishAccountTypeDTO::getGhId, Function.identity()));
        // 删除文章
        List<JSONObject> alarmList = new ArrayList<>();
        for (String publishContentId : publishContentIds) {
            try {
                String pushId = publishPushIdMap.get(publishContentId);
                if (!StringUtils.hasText(pushId)) {
                    deleteFailAlarmAdd(alarmList, publishContentId, "无推送记录", 0);
                    continue;
                }
                PublishGzhPushDTO publishGzhPushDTO = pushDTOMap.get(pushId);
                if (Objects.isNull(publishGzhPushDTO)) {
                    deleteFailAlarmAdd(alarmList, publishContentId, "无推送记录", 0);
                    continue;
                }
                String publishAccountId = pushAccountMap.get(pushId);
                String ghId = publishAccountMap.get(publishAccountId);
                List<PublishGzhPushContentRelDTO> relList = groupPushRelMap.get(pushId).stream()
                        .sorted(Comparator.comparing(PublishGzhPushContentRelDTO::getId)).collect(Collectors.toList());
                int index = 1;
                for (int i = 0; i < relList.size(); i++) {
                    if (relList.get(i).getPublishContentId().equals(publishContentId)) {
                        index = i + 1;
                        break;
                    }
                }
                if (!publishGzhPushDTO.getPushType().equals(PushTypeEnum.AUTO_GROUP_PUBLISH.getVal())) {
                    PushTypeEnum pushTypeEnum = PushTypeEnum.from(publishGzhPushDTO.getPushType());
                    deleteFailAlarmAdd(alarmList, publishContentId, "推送类型为" + pushTypeEnum.getDescription(), index);
                    continue;
                }
                String groupPushMsgId = pushIdMap.get(pushId);
                if (!StringUtils.hasText(groupPushMsgId)) {
                    deleteFailAlarmAdd(alarmList, publishContentId, "无推送MsgId", index);
                    continue;
                }
                LongArticleAuditDelete delete = new LongArticleAuditDelete();
                delete.setGhId(ghId);
                delete.setMsgId(groupPushMsgId);
                delete.setPushId(pushId);
                delete.setPushType(publishGzhPushDTO.getPushType());
                delete.setPublishContentId(publishContentId);
                delete.setIndex(index);
                delete.setStatus(StatusEnum.ZERO.getCode());
                delete.setCreateTimestamp(System.currentTimeMillis());
                longArticleAuditDeleteRepository.save(delete);
            } catch (Exception e) {
                log.error("buildArticleAuditDelete error", e);
            }
        }
        deleteFailAlarm(alarmList, accountTypeMap);
    }

    private void deleteFailAlarmAdd(List<JSONObject> alarmList, String publishContentId, String errMsg, Integer index) {
        JSONObject alarm = new JSONObject();
        alarm.put("publishContentId", publishContentId);
        alarm.put("errMsg", errMsg);
        alarm.put("index", index);
        alarmList.add(alarm);
    }

    /**
     * 删除审核不通过，已发布文章
     */
    @XxlJob("articleVideoDelete")
    public ReturnT<String> articleVideoDelete(String param) {
        List<LongArticleAuditDelete> dealList = longArticleAuditDeleteRepository.getByStatus(ArticleDeleteStatusEnum.WAITING.getCode());
        List<PublishAccountTypeDTO> accountTypeList = aigcBaseMapper.getAccountTypeList();
        Map<String, PublishAccountTypeDTO> accountTypeMap = accountTypeList.stream()
                .collect(Collectors.toMap(PublishAccountTypeDTO::getGhId, Function.identity()));
        List<JSONObject> alarmList = new ArrayList<>();
        for (LongArticleAuditDelete delete : dealList) {
            try {
                if (Objects.equals(delete.getPushType(), PushTypeEnum.AUTO_GROUP_PUBLISH.getVal())) {
                    // 获取token
                    String token = wxAccessTokenRemoteService.getAccessToken(delete.getGhId());
                    // 删除文章
                    RequestResult<String> result = wxArticleDeleteService.deleteArticle(token, delete.getMsgId(), delete.getIndex());
                    if (result.isSuccess()) {
                        delete.setStatus(ArticleDeleteStatusEnum.SUCCESS.getCode());
                    } else {
                        delete.setStatus(ArticleDeleteStatusEnum.FAIL.getCode());
                        delete.setFailReason(result.getFailReason());
                    }
                } else {
                    deleteFailAlarmAdd(alarmList, delete.getPublishContentId(), "非自动群发", delete.getIndex());
                    delete.setStatus(ArticleDeleteStatusEnum.SUCCESS.getCode());
                }
                delete.setFinishTimestamp(System.currentTimeMillis());
                longArticleAuditDeleteRepository.save(delete);
            } catch (Exception e) {
                log.error("articleVideoDelete ghId:{} error", delete.getGhId(), e);
            }
        }
        deleteFailAlarm(alarmList, accountTypeMap);
        return ReturnT.SUCCESS;
    }

    private void deleteFailAlarm(List<JSONObject> alarmList,
                                 Map<String, PublishAccountTypeDTO> accountTypeMap) {
        List<String> publishContentIds = alarmList.stream().map(o -> o.getString("publishContentId"))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(publishContentIds)) {
            return;
        }
        List<PublishContentDTO> publishContents = aigcBaseMapper.getPublishContentByIdIn(publishContentIds);
        Map<String, PublishContentDTO> publishContentMap = publishContents.stream()
                .collect(Collectors.toMap(PublishContentDTO::getId, Function.identity()));
        List<String> publishAccountIds = publishContents.stream().map(PublishContentDTO::getPublishAccountId)
                .collect(Collectors.toList());
        List<PublishAccount> publishAccounts = publishAccountRepository.getByIdIn(publishAccountIds);
        Map<String, PublishAccount> publishAccountMap = publishAccounts.stream()
                .collect(Collectors.toMap(PublishAccount::getId, Function.identity()));
        List<FeishuTableDTO.Column> columns = buildArticleDeleteFailColumns();
        List<JSONObject> rows = new ArrayList<>();
        for (JSONObject alarm : alarmList) {
            String publishContentId = alarm.getString("publishContentId");
            PublishContentDTO publishContent = publishContentMap.get(publishContentId);
            String publishTime = DateUtils.timestampToYMDStr(publishContent.getPublishTimestamp() / 1000, "yyyyMMdd");
            PublishAccount publishAccount = publishAccountMap.get(publishContent.getPublishAccountId());
            PublishAccountTypeDTO accountTypeDTO = accountTypeMap.get(publishAccount.getGhId());
            String accountType = Objects.nonNull(accountTypeDTO) ? accountTypeDTO.getAccountSourceName() : "未知";

            JSONObject row = new JSONObject();
            row.put("account_name", publishAccount.getName());
            row.put("gh_id", publishAccount.getGhId());
            row.put("account_type", accountType);
            row.put("publish_time", publishTime);
            row.put("index", String.valueOf(alarm.getInteger("index")));
            row.put("title", publishContent.getTitle());
            row.put("err_msg", alarm.getString("errMsg"));
            rows.add(row);
        }
        FeishuTableDTO tableDTO = FeishuTableDTO.createTable("文章删除失败", columns, rows);
        JSONObject content = JSONObject.parseObject(JSONObject.toJSONString(tableDTO));
        // 无法自动删除 发送飞书通知 人工删除
        JSONObject bodyParam = new JSONObject();
        bodyParam.put("msg_type", "interactive");
        bodyParam.put("card", content);
        FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.ARTICLE_DELETE.getRobotId(), bodyParam);
    }

    private List<FeishuTableDTO.Column> buildArticleDeleteFailColumns() {
        List<FeishuTableDTO.Column> columns = new ArrayList<>();
        FeishuTableDTO.Column accountNameColumn = FeishuTableDTO.createFeishuColumns(
                FieshuTableColumnDataTypeEnum.TEXT.getType(), "account_name", "账号名称", null);
        columns.add(accountNameColumn);
        FeishuTableDTO.Column accountGhIdColumn = FeishuTableDTO.createFeishuColumns(
                FieshuTableColumnDataTypeEnum.TEXT.getType(), "gh_id", "账号ghId", null);
        columns.add(accountGhIdColumn);
        FeishuTableDTO.Column accountTypeColumn = FeishuTableDTO.createFeishuColumns(
                FieshuTableColumnDataTypeEnum.TEXT.getType(), "account_type", "账号来源", null);
        columns.add(accountTypeColumn);
        FeishuTableDTO.Column publishTimeColumn = FeishuTableDTO.createFeishuColumns(
                FieshuTableColumnDataTypeEnum.TEXT.getType(), "publish_time", "发布日期", null);
        columns.add(publishTimeColumn);
        FeishuTableDTO.Column indexColumn = FeishuTableDTO.createFeishuColumns(
                FieshuTableColumnDataTypeEnum.TEXT.getType(), "index", "位置", null);
        columns.add(indexColumn);
        FeishuTableDTO.Column titleColumn = FeishuTableDTO.createFeishuColumns(
                FieshuTableColumnDataTypeEnum.TEXT.getType(), "title", "标题", null);
        columns.add(titleColumn);
        FeishuTableDTO.Column errMsgColumn = FeishuTableDTO.createFeishuColumns(
                FieshuTableColumnDataTypeEnum.TEXT.getType(), "err_msg", "原因", null);
        columns.add(errMsgColumn);
        return columns;
    }

    public void titleDangerFindDelete(ArticleDangerFindDeleteParam param) {
        String titleMd5 = Md5Util.encoderByMd5(param.getTitle());
        // 根据标题查找已发布文章
        List<Article> articleList = articleRepository.getByTitleMd5(titleMd5);
        List<String> ghIds = articleList.stream().map(Article::getGhId).distinct().collect(Collectors.toList());
        List<PublishAccount> publishAccountList = publishAccountRepository.getAllByGhIdIn(ghIds);
        List<PublishContentParam> publishContentParamList = publishAccountList.stream().map(account -> {
            PublishContentParam item = new PublishContentParam();
            item.setTitle(param.getTitle());
            item.setPublishAccountId(account.getId());
            return item;
        }).collect(Collectors.toList());
        List<PublishContentDTO> publishContents = new ArrayList<>();
        for (List<PublishContentParam> partitions : Lists.partition(publishContentParamList, 100)) {
            publishContents.addAll(aigcBaseMapper.getPublishContentByTitle(partitions));
        }
        if (CollectionUtils.isEmpty(publishContents)) {
            return;
        }
        // 处理视频内容池内容
        List<PublishContentDTO> videoPoolContents = publishContents.stream()
                .filter(o -> Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal()))
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(videoPoolContents)) {
            for (PublishContentDTO videoPoolContent : videoPoolContents) {
                longArticleBaseMapper.updateVideoPoolContentBad(videoPoolContent.getSourceId());
            }
        }
        // 查找该生成内容下所有已发布内容
        List<String> sourceIds = publishContents.stream().map(PublishContentDTO::getSourceId).distinct().collect(Collectors.toList());
        publishContents = aigcBaseMapper.getPublishContentBySourceIdIn(sourceIds);

        // 根据已发布文章查找推送id 并删除
        List<String> publishContentIds = publishContents.stream().map(PublishContentDTO::getId).collect(Collectors.toList());
        buildArticleAuditDelete(publishContentIds);
        buildArticleUnsafeTitle(titleMd5, param.getTitle());
    }

    private void buildArticleUnsafeTitle(String titleMd5, String title) {
        ArticleUnsafeTitle unsafeTitle = new ArticleUnsafeTitle();
        unsafeTitle.setTitle(title);
        unsafeTitle.setTitleMd5(titleMd5);
        unsafeTitle.setStatus(StatusEnum.ONE.getCode());
        unsafeTitle.setCreateTimestamp(System.currentTimeMillis());
        articleUnsafeTitleRepository.save(unsafeTitle);
    }

}
